@@ -2,8 +2,7 @@
 # Use the `scipy.linalg` namespace for importing the functions
 # included below.
 
-import warnings
-from . import _special_matrices
+from scipy._lib.deprecation import _sub_module_deprecation
 
 __all__ = [  # noqa: F822
     'tri', 'tril', 'triu', 'toeplitz', 'circulant', 'hankel',
@@ -18,13 +17,6 @@ def __dir__():
 
 
 def __getattr__(name):
-    if name not in __all__:
-        raise AttributeError(
-            "scipy.linalg.special_matrices is deprecated and has no attribute "
-            f"{name}. Try looking in scipy.linalg instead.")
-
-    warnings.warn(f"Please use `{name}` from the `scipy.linalg` namespace, the"
-                  " `scipy.linalg.special_matrices` namespace is deprecated.",
-                  category=DeprecationWarning, stacklevel=2)
-
-    return getattr(_special_matrices, name)
+    return _sub_module_deprecation(sub_package="linalg", module="special_matrices",
+                                   private_modules=["_special_matrices"], all=__all__,
+                                   attribute=name)
